kmean::kmean(int ID,double X, double Y, double Z,unsigned int N)
{
	id = ID;
	x = X;
	y = Y;
	z = Z;
	r = gsl_vector_calloc(N);
	n = N;
	for(int i=0;i<N;i++)
		gsl_vector_set_all(r,0);
}

kmean::~kmean()
{
	gsl_vector_free(r);
}

int kmean::move(double X, double Y, double Z)
{
	x = X;
	y = Y;
	z = Z;
	return 0;
}

int kmean::calculate_distances(gsl_matrix *MatrixIN, int N, gsl_matrix *distances)
{
	double a1,a2,a3;
	for(int i=0; i<N; i++)
	{
		a1 = gsl_matrix_get(MatrixIN,i,1);
		a2 = gsl_matrix_get(MatrixIN,i,2);
		a3 = gsl_matrix_get(MatrixIN,i,3);

		gsl_matrix_set(distances,id,i,distance(a1,a2,a3,x,y,z));


	}
	return 0;
}

int kmean::RR(void)
{
	for (int i=0; i<n; i++)
		printf("%d - %g\n",i,gsl_vector_get(r,i));
	return 0;
}
char * kmean::whereis(void)
{
	int n=sprintf(buffer,"%10.5f\t%10.5f\t%10.5f\n",x,y,z);
	return buffer;
}

int kmean::set_r(int index, bool included)
{
	gsl_vector_set(r,index,included);
	return 0;
}

int check_distances(gsl_matrix* distances, int numberOfWorldElements, kmean **means)
{
	//Checks the distances between each element in the world
	//and the means, finds the nearest mean, registers the
	//element in that mean's responsibility list.

	//int row = k->II() - 1;
	//gsl_vector_view r = gsl_matrix_row(distances,row);
	//gsl_vector_fprintf(stdout,&r.vector,"%g");
	
	gsl_vector_view v;
	int mindex = -1;
	for(int i=0;i<numberOfWorldElements;i++)
	{
		//We're going to find the minimum (ie, nearest) values 
		//in the "distances" matrix cols. The index of the minimum
		//will give us the mean id we're looking for!
		
		//First we're slicing the cols for each element
		v = gsl_matrix_column(distances,i);
		
		//setting every means' responsibility to 0 for the data in question
		for(int j=0;j<dK;j++)
			means[j]->set_r(i,0);

		//getting the index for minima:
		mindex = gsl_vector_min_index(&v.vector);
		//printf ("%i \t %i\n",i,mindex);

		//setting the responsibility of the minima yielding means
		//for the particular element:
		means[mindex]->set_r(i,1);
	}

	
	return 0;
}
int kmean::moveCoM(gsl_matrix* MatrixIN)
{
	//moves the mean to the center of mass of its
	//responisibility data
	
	double CoM_x = 0.0;
	double CoM_y = 0.0;
	double CoM_z = 0.0;
	int r_tot = 0;
	int ri;

	for (int i=0; i<n; i++)
	{
		ri = int(gsl_vector_get(r,i));
		CoM_x += ri * gsl_matrix_get(MatrixIN,i,1);
		CoM_y += ri * gsl_matrix_get(MatrixIN,i,2);
		CoM_z += ri * gsl_matrix_get(MatrixIN,i,3);
		r_tot += ri;
	}
	if(!_quiet) printf("Mean %d has %d responsibilities over a total of %d responsibility\n",id,r_tot,n);
	if(r_tot>0)
	{
		CoM_x /= r_tot;
		CoM_y /= r_tot;
		CoM_z /= r_tot;
		this->move(CoM_x, CoM_y, CoM_z);
	}
	r_tott = r_tot;
	return 0;
}

double distance(double a1, double a2,double a3,double b1,double b2,double b3)
{
	//distance function.
	
	return sqrt(pow((a1-b1),2) + pow((a2-b2),2) + pow((a3-b3),2));
}
