int M_InitializeEST ()
{
	//Random Number Initializer-------BEGIN-------//
	gsl_rng_env_setup();
	
	r = gsl_rng_alloc (gsl_rng_mt19937);

	if(seed == 0) seed = time(NULL);
	gsl_rng_set(r,seed);
	if(!_quiet)
	{
		printf ("generator type: %s\n", gsl_rng_name (r));
		printf ("seed = %lu\n", seed);
		//printf ("first value = %lu\n", gsl_rng_get (r));
	}
	
	//Random Number Initializer-------END---------//
	
	return 0;
}
int M_RandomizerEST (int N, gsl_matrix *MatrixIN, double min=0.0, double max=10.0 )
{
	/*
	 * Randomizes a Nx4 matrix' last 2.,3.,4. columns
	 * using values in the range [min,max]. Sets the first column
	 * as the row number.
	 * 
	 * Emre S. Tasci <E.Tasci@tudelft.nl>
	 * http: * www.emresururi.com/physics/
	 * 
	 * 28/11/2007
	*/
	int i, j; 
	
	for (i = 0; i < N; i++)
	{
		gsl_matrix_set(MatrixIN,i,0,i);
		for (j = 1; j < 4; j++)
			//gsl_matrix_set (MatrixIN, i, j, i+ 0.2*j);
			gsl_matrix_set (MatrixIN, i, j, gsl_rng_uniform(r)*(max-min)+min);
	}
	
	return 0;
}

int M_PrintEST(gsl_matrix *MatrixIN,int N,int J=4,bool firstvalint=true)
{
	//Prints the given [NxJ] matrix  in a readable format. 
	printf("\n\n");
	int i,j;
	if(firstvalint)
	for (i = 0; i < N; i++)
	{
		printf ("m(%2d,%2d) = %2d\t", i, 0, int(gsl_matrix_get (MatrixIN, i, 0)));
		for (j = 1; j < J; j++)
			printf ("m(%2d,%2d) = %12.9f\t", i, j, gsl_matrix_get (MatrixIN, i, j));
		printf("\n");
	}
	else
	for (i = 0; i < N; i++)
	{
		for (j = 0; j < J; j++)
			printf ("m(%2d,%2d) = %12.9f\t", i, j, gsl_matrix_get (MatrixIN, i, j));
		printf("\n");
	}
	return 0;

}

int M_Write2FileEST(string baseFileName, gsl_matrix * MatrixIN, int N, bool NN=true)
{
	/*
	 * Writes the given matrix to filename1 using gsl_matrix_fprintf
	 * and to filename2 using a more readable format
	 *
	 * If bool NN is true, also writes the first column into filename2
	 *
	 * Emre S. Tasci <E.Tasci@tudelft.nl>
	 * http: * www.emresururi.com/physics/
	 * 
	 * 28/11/2007
	*/
	FILE * pFile1;
	FILE * pFile2;
	FILE * pFile3;

	int i,j;


	pFile1 = fopen(extension_changer_adderEST(baseFileName,"world.dat").c_str(),"w");
	pFile2 = fopen(extension_changer_adderEST(baseFileName,"world.txt").c_str(),"w");

	gsl_matrix_fprintf(pFile1, MatrixIN, "%g");
	
	//I am using two "identical" loops instead of one with the If statement
	//sitting on top of the [(i,0) print] procedure because of optimization
	//
	//(1 check of NN instead of N checks)

	//Writing the relevant information to a seperate file so that if needed,
	//the run can be repeated at a later time... ---------------------------
	time_t rawtime;	
	struct tm * timeinfo;
  	time ( &rawtime );
  	timeinfo = localtime ( &rawtime );
	pFile3 = fopen(extension_changer_adderEST(baseFileName,"report.txt").c_str(),"w");
	fprintf (pFile3,"generator type: %s\n", gsl_rng_name (r));
	fprintf (pFile3,"seed = %lu\n", seed);
	fprintf (pFile3,"number of data = %i\n", dN);
	fprintf (pFile3,"date and time : %s\n", asctime(timeinfo));
	fclose(pFile3);
	//----------------------------------------------------------------------

	if(NN)
	{
		for (i = 0; i < N; i++)
		{
			fprintf (pFile2,"%2d\t", int(gsl_matrix_get (MatrixIN, i, 0)));
			for (j = 1; j < 4; j++)
				fprintf (pFile2,"%12.9f\t", gsl_matrix_get (MatrixIN, i, j));
			fprintf(pFile2,"\n");
		}
	}
	else
	{
		for (i = 0; i < N; i++)
		{
			for (j = 1; j < 4; j++)
				fprintf (pFile2,"%6.3f\t", gsl_matrix_get (MatrixIN, i, j));
				//fprintf (pFile2,"%12.9f\t", gsl_matrix_get (MatrixIN, i, j));
			fprintf(pFile2,"\n");
		}
	}

	fclose(pFile1);
	fclose(pFile2);
	return 0;
}
	
	

int Finalize(void)
{
	gsl_rng_free(r);
	return 0;
}
